var
   fs     = require('fs'),
   path   = require('path'),
   global = (function() { return this || (1,eval)('this') }());

module.exports = function(grunt) {
   grunt.registerMultiTask('rjs-build', 'build packages', function(){
      var done = this.async();

      if (!this.data.root){
         throw new Error('Не задана опция "root"! Укажите где находится корень вашего приложения!');
      }
      if (!this.data.out){
         throw new Error('Не задана опция "out"! Укажите расположение выходного файла относительно корня приложения!');
      }
      if (!this.data.name){
         throw new Error('Не задана опция "name"! Укажите имя главного модуля вашего приложения!');
      }

      this.data.ws        = this.data.ws || 'ws';
      this.data.resources = this.data.ws || 'resources';
      this.data.paths     = this.data.paths || {};
      this.data.include   = this.data.include || [];

      var
         wsRoot           = path.resolve(this.data.root, this.data.ws) + '/',
         resourceRoot     = path.resolve(this.data.root, this.data.resources),
         contentsJSONPath = path.join(resourceRoot, 'contents.json');

      //Конфигурация ядра
      global.wsConfig = {
         appRoot: this.data.root,
         wsRoot:  this.data.ws + '/',
         resourceRoot: path.resolve(this.data.root, this.data.resources)
      };

      //Подключаем ядро
      require(path.join(wsRoot, 'lib/core.js'));

      //Грузим оглавление если можем
      if (fs.existsSync(contentsJSONPath)){
         $ws.helpers.loadContents(require(contentsJSONPath));
      }
      else{
         grunt.log.warn('Файл ' + contentsJSONPath + ' не найден. Игнорируем загрузку оглавления.');
      }

      var requirejs = require('requirejs');

      //Стрырил логилку у grunt-contrib-requirejs
      requirejs.define('node/print', [], function() {
         return function print(msg) {
            if (msg.substring(0, 5) === 'Error') {
               grunt.log.errorlns(msg);
               grunt.fail.warn('RequireJS failed.');
            } else {
               grunt.log.oklns(msg);
            }
         };
      });

      //Конфигурирование requirejs
      var nodeRequire = require;
      global.require = requirejs;
      global.requirejs = requirejs;
      var createRequirejsConfig = nodeRequire(path.join(wsRoot, 'ext/requirejs/config.js'));
      global.require = nodeRequire;

      //Получаем конфиг для requirejs
      var cfg = createRequirejsConfig(this.data.root, this.data.ws, this.data.resources, {
         baseUrl: this.data.root,     // папка где находятся все js файлы
         name: this.data.name,        // название файла запускающего приложение
         include: this.data.include,  // инклюды, добавляемые в пачку
         out: this.data.out,          // имя файла на выходе
         separateCSS : true,          // выделям отдельный файл для css или нет
         logLevel: 0                  // уровень логирования
      });

      //Прописываем дополнительные paths переданные в опциях таска
      Object.keys(this.data.paths).forEach(function(key){
         cfg.paths[key] = this.data.paths[key];
      }.bind(this));

      //Билдим
      requirejs.optimize(cfg, done);
   });
};